import greenfoot.*;
import java.util.ArrayList;
import java.util.List;
public class Field extends World
{
    public final int cellSize;
    private Mover m;
    private AI ai;
    public ArrayList<PathFinder> pathFindQue;
    public Box b;
    public GreenfootImage image;
    public MouseEffect e;
    public boolean attack;
    public Field()
    {    
        super(920, 660, 1);
        cellSize = 20;
        m = new Mover(this);
        ai = new AI(this);
        e = new MouseEffect();
        spawnHuts(1);
        spawnUnits(10);
        spawnWalls(400);//trees
        pathFindQue = (ArrayList)getObjects(PathFinder.class);//initialize
        setPaintOrder(Units.class);
        setActOrder(Fog.class);
        image = getBackground();
        addObject(e,0,0);
        attack = false;
        //spawnFog();
    }
    public void act()
    {
        m.act();
        //ai.act();
        if(!pathFindQue.isEmpty())
        {
            pathFindQue.get(0).makePath();
            pathFindQue.remove(0);
        }
        if(leftClicked() && Greenfoot.mousePressed(null) && !attack)
        {
            deselect();//deselect everything before adding box
            spawnBox();
        }
        else if(leftClicked() && Greenfoot.mouseClicked(null) && !attack)
        {
            b.select();//select everything before removing box
            removeBox();
        }
        else if(leftClicked() && Greenfoot.mouseClicked(null) && attack)//issue attack command
        {
            moveEffect(attack);
            attack = false;//only issue once
        }
        if(rightClicked() && Greenfoot.mouseClicked(null) && attack)//right click to cancle attack command
        {
            attack = false;
        }
        else if(rightClicked() && Greenfoot.mouseClicked(null) && !attack)//issue move command
        {
            moveEffect(attack);
        }
        if(Greenfoot.isKeyDown("a") && b == null)//cannot issue attack command when box in world
        {
            attack = true;
        }
    }
    public void joinQue(PathFinder f)
    {
        pathFindQue.add(f);
    }
    public boolean leftClicked()
    {
        if(Greenfoot.getMouseInfo() == null) return false;
        return Greenfoot.getMouseInfo().getButton() == 1;
    }
    public boolean rightClicked()
    {
        if(Greenfoot.getMouseInfo() == null) return false;
        return Greenfoot.getMouseInfo().getButton() == 3;
    }
    public void spawnBox()
    {
        int x = Greenfoot.getMouseInfo().getX();
        int y = Greenfoot.getMouseInfo().getY();
        b = new Box(x, y);
        addObject(b, x, y);
    }
    public void removeBox()
    {
        removeObject(b);
        b = null;
    }
    public void deselect()
    {
        List<Selectable> list = getObjects(Selectable.class);
        int size = list.size();
        for(int n = 0; n < size; n ++)
        {
            list.get(n).deselect();
        }
    }
    public void moveEffect(boolean b)
    {
        int x = Greenfoot.getMouseInfo().getX();
        int y = Greenfoot.getMouseInfo().getY();
        if(b)//attack
        {
            e.setState(1);
            List<Selectable> list = getObjects(Selectable.class);
            int size = list.size();
            for(int n = 0; n < size; n ++)
            {
                if(list.get(n).selected())
                {
                    list.get(n).attackMove(x, y);
                }
            }
        }
        else//regular move
        {
            e.setState(0);
            List<Selectable> list = getObjects(Selectable.class);
            int size = list.size();
            for(int n = 0; n < size; n ++)
            {
                if(list.get(n).selected())
                {
                    list.get(n).move(x, y);
                }
            }
        }
        e.resetTimer();
        e.setLocation(x, y);
    }
    //important stuff on top
    //----------------------------------------------------------------------------
    
    public void addBlood(int x, int y)
    {
        image.drawImage(new GreenfootImage("BloodSplat.png"), x, y);
        setBackground(image);
    }
    public void spawnFog()
    {
        for(int x = 5; x < getWidth(); x += 10)
        {
            for(int y = 5; y < getHeight(); y += 10)
            {
                Fog f = new Fog();
                addObject(f, x, y);
            }
        }
    }
    public void spawnWalls(int amount)
    {
        for(int n = 0; n < amount; n ++)
        {
            int x = Greenfoot.getRandomNumber(getWidth())/cellSize;
            int y = Greenfoot.getRandomNumber(getHeight())/cellSize;
            while(!getObjectsAt(x*cellSize + cellSize/2, y*cellSize + cellSize/2, Actor.class).isEmpty())
            {
                x = Greenfoot.getRandomNumber(getWidth())/cellSize;
                y = Greenfoot.getRandomNumber(getHeight())/cellSize;
            }
            addObject(new Wall(), x*cellSize + cellSize/2, y*cellSize + cellSize/2);
        }
    }
    public void spawnUnits(int n)//fill empty with Unitss
    {
        int amount = 0;
        int x = Greenfoot.getRandomNumber(getWidth()/cellSize);
        int y = Greenfoot.getRandomNumber(getHeight()/cellSize);
        while(amount < n)
        {
            while(!getObjectsAt(x*cellSize + cellSize/2, y*cellSize + cellSize/2, Actor.class).isEmpty())
            {
                x = Greenfoot.getRandomNumber(getWidth()/cellSize);
                y = Greenfoot.getRandomNumber(getHeight()/cellSize);
            }
            addObject(new Units(0), x*cellSize + cellSize/2, y*cellSize + cellSize/2);
            amount ++;
        }
    }
    public int getUnits()
    {
        return getObjects(Units.class).size();
    }
    public void spawnHuts(int amount)
    {
        int n = 0;
        int f = 0;
        int x = Greenfoot.getRandomNumber(getWidth()/cellSize - 1);
        int y = Greenfoot.getRandomNumber(getHeight()/cellSize - 1);
        while(amount > n)
        {
            while(!getObjectsAt(x*cellSize + cellSize/2, y*cellSize + cellSize/2, Actor.class).isEmpty() || !getObjectsAt((x+1)*cellSize + cellSize/2, (y+1)*cellSize + cellSize/2, Actor.class).isEmpty() || !getObjectsAt((x+1)*cellSize + cellSize/2, y*cellSize + cellSize/2, Actor.class).isEmpty() || !getObjectsAt(x*cellSize + cellSize/2, (y+1)*cellSize + cellSize/2, Actor.class).isEmpty())
            {
                x = Greenfoot.getRandomNumber(getWidth()/cellSize - 1);
                y = Greenfoot.getRandomNumber(getHeight()/cellSize - 1);
            }
            spawnHut(x, y, f);
            n ++;
            if(f == 0)
            {
                f = 1;
            }
            else
            {
                f = 0;
            }
        }
    }
    public void spawnHut(int x, int y, int faction)//x,y is top left corner(x,y is the gridX and gridY)
    {
        addObject(new Hut(faction), x*cellSize + cellSize, y*cellSize + cellSize);
    }
}
