import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
public class PathFinder extends SmoothMover
{
    public ArrayList<Node> openList;
    public ArrayList<Node> closedList;
    public ArrayList<Point> path;//list of points that lead to gridDestination
    public int pathIndex;//how far along the path is dude?
    public boolean havePath;//does dude have a path to gridDestination?
    public Point destination;//where dude is heading//it should be the actual point not the grid point
    
    public double speed;
    public PathFinder()
    {
        speed = 0.5;
        pathIndex = 0;
        havePath = false;
    }
    public void addedToWorld(World w)
    {
        destination = new Point(getX(), getY());//destination is where they currently are(not heading anywhere)
        currentX = getX();
        currentY = getY();
    }
    public int getGridX()//get the grid's x (ie 1 becomes 1/cellSize)
    {
        Field f = (Field)getWorld();
        return getX()/f.cellSize;
    }
    public int getGridY()//get the grid's y
    {
        Field f = (Field)getWorld();
        return getY()/f.cellSize;
    }
    public Point toGrid(Point p)//returns the grid equvilant of p
    {
        Field f = (Field)getWorld();
        return new Point(p.getX()/f.cellSize, p.getY()/f.cellSize);
    }
    public Point toActual(Point p)//returns the actual equvilant of p
    {
        Field f = (Field)getWorld();
        return new Point(p.getX()*f.cellSize + f.cellSize/2, p.getY()*f.cellSize + f.cellSize/2);
    }
    public void move()
    {
        Point gridDestination = toGrid(destination);
        if(!gridDestination.is(new Point(getGridX(), getGridY())) && havePath)//if not at gridDestination, do stuff
        {
            if(pathIndex > 0 && new Point(getGridX(), getGridY()).is(path.get(pathIndex)))//if not at (final)gridDestination, and has reached where dude is currently trying to reach, then move on to the next step
            {
                pathIndex--;//Next spot
            }
            else if(pathIndex >= 0)
            {
                turnTowards(toActual(path.get(pathIndex)).getX(), toActual(path.get(pathIndex)).getY());
                move(speed);
            }
        }
        else if(havePath && !destination.is(new Point(getX(), getY())))//if at gridDestination, then try and get to actual destination
        {
            turnTowards(destination.getX(), destination.getY());
            move(speed);
        }
    }
    public void act() 
    {
        if(Greenfoot.getMouseInfo() != null)
        {
            if(Greenfoot.mousePressed(null))
            {
                destination.setPoint(Greenfoot.getMouseInfo().getX(), Greenfoot.getMouseInfo().getY());
                Field w = (Field)getWorld();
                w.joinQue(this);
                havePath = false;
            }
        }
    }    
    
    public void makePath()
    {
        Point gridDestination = toGrid(destination);
        
        havePath = false;//just started charting path
        openList = new ArrayList<Node>();
        closedList = new ArrayList<Node>();//reset open and closed list
        Node startNode = new Node(new Point(getGridX(), getGridY()), null, gridDestination);//the first node to be scan
        openList.add(startNode);
        for(int count = 0; count < 2000 && !havePath; count++)//while dont have a path and looped less then 2000 times
        {
            if(openList.isEmpty())
            {
                //destination = new Point(getX(),getY());
                return;
            }
                
            int lowest = 0;//Index for node that has lowest F score
            for(int n = 1; n < openList.size(); n++)//look through openList and gets the Node that has the lowest F score
            {
                if (openList.get(n).getF() < openList.get(lowest).getF()) // Check to see if score of this Node is the new lowest
                    lowest = n; // This lowest one found so far
            }
            
            checkSurroundingNodes(lowest);//checks the node surrounding the node wer looking at
            closedList.add(openList.get(lowest));
            openList.remove(lowest);//openList.get(lowest) has been checked so we put it in closed list
            //---------------------------------------------------------------------------------------------
            if(isInClosed(gridDestination))//if a node in the closed list is located on the gridDestination
            {
                havePath = true;//breaks out of the loop
            }
            
        }
        if(havePath)
        {
            path = new ArrayList<Point>(); // Refresh the path to take
            int i = indexInClosed(gridDestination);
            Node node = closedList.get(i);//start looking at the gridDestination and trace parent back to start
            while(node != null) // Loop till back to beginning, parent at start = null
            {
                path.add(node.position); // Add point
                node = node.getParent(); // Move to that Node's parent
            }
            pathIndex = path.size() - 2; // When start to move, starting at beginning
        }
    }
    
    public void treat(Point p, Node theNode, Point destination)
    {
        Node node = new Node(p, theNode, destination);
        if(isInOpen(node))//if node is already in openList
        {
            int i = indexInOpen(node);//i is the index where node is located in the openList
            if(openList.get(i).biggerG(theNode))//g value is lower with openList.get(lowest) as its parent
            {
                openList.get(i).adopt(theNode);//replace the node in the list with a new node with its parent set to the currentNode wer looking at
            }
        }
        else if(!isInClosed(node))//if not in open list and not in closed, then add to open list
        {
            openList.add(node);
        }
    }
    
    public void checkSurroundingNodes(int index)
    {
        Node theNode = openList.get(index);//the center node
        Field w = (Field)getWorld();
        Point gridDestination = toGrid(destination);
        
        Point left = new Point(openList.get(index).position.getX() - 1, openList.get(index).position.getY());//checking the point on the left
        if(left.getX() >= 0)//in bounds of grid
        {
            if(w.getObjectsAt(toActual(left).getX(), toActual(left).getY(), Wall.class).isEmpty() || left.is(gridDestination))//if no wall exsist there or its the gridDestination
            {
                treat(left, theNode, gridDestination);
            }
        }
        
        Point right = new Point(openList.get(index).position.getX() + 1, openList.get(index).position.getY());
        if(right.getX() < getWorld().getWidth()/w.cellSize) // in bounds of grid
        {
            if(w.getObjectsAt(toActual(right).getX(), toActual(right).getY(), Wall.class).isEmpty() || right.is(gridDestination)) // unblocked
            {
                treat(right, theNode, gridDestination);
            }
        }
        
        Point up = new Point(openList.get(index).position.getX(), openList.get(index).position.getY()-1);
        if(up.getY() >= 0) // in bounds of grid
        {
            if(w.getObjectsAt(toActual(up).getX(), toActual(up).getY(), Wall.class).isEmpty() || up.is(gridDestination)) // unblocked
            {
                treat(up, theNode, gridDestination);
            }
        }
        
        Point down = new Point(openList.get(index).position.getX(), openList.get(index).position.getY()+1);
        if(down.getY() < getWorld().getHeight()/w.cellSize) // in bounds of grid
        {
            if(w.getObjectsAt(toActual(down).getX(), toActual(down).getY(), Wall.class).isEmpty() || down.is(gridDestination)) // unblocked
            {
                treat(down, theNode, gridDestination);
            }
        }
        
        Point downLeft = new Point(openList.get(index).position.getX()-1, openList.get(index).position.getY()+1);
        if(downLeft.getX() >= 0 && downLeft.getY() < getWorld().getHeight()/w.cellSize) // in bounds of grid
        {
            if((w.getObjectsAt(toActual(downLeft).getX(), toActual(downLeft).getY(), Wall.class).isEmpty() || downLeft.is(gridDestination)) && (w.getObjectsAt(toActual(down).getX(), toActual(down).getY(), Wall.class).isEmpty() && w.getObjectsAt(toActual(left).getX(), toActual(left).getY(), Wall.class).isEmpty())) // unblocked
            {
                treat(downLeft, theNode, gridDestination);
            }
        }
        
        Point downRight = new Point(openList.get(index).position.getX()+1, openList.get(index).position.getY()+1);
        if(downRight.getX() < getWorld().getWidth()/w.cellSize && downRight.getY() < getWorld().getHeight()/w.cellSize) // in bounds of grid
        {
            if((w.getObjectsAt(toActual(downRight).getX(), toActual(downRight).getY(), Wall.class).isEmpty() || downRight.is(gridDestination)) && (w.getObjectsAt(toActual(down).getX(), toActual(down).getY(), Wall.class).isEmpty() && w.getObjectsAt(toActual(right).getX(), toActual(right).getY(), Wall.class).isEmpty())) // unblocked
            {
                treat(downRight, theNode, gridDestination);
            }
        }
        
        Point upRight = new Point(openList.get(index).position.getX()+1, openList.get(index).position.getY()-1);
        if(upRight.getX() < getWorld().getWidth()/w.cellSize && upRight.getY() >= 0) // in bounds of grid
        {
            if((w.getObjectsAt(toActual(upRight).getX(), toActual(upRight).getY(), Wall.class).isEmpty() || upRight.is(gridDestination)) && (w.getObjectsAt(toActual(up).getX(), toActual(up).getY(), Wall.class).isEmpty() && w.getObjectsAt(toActual(right).getX(), toActual(right).getY(), Wall.class).isEmpty())) // unblocked
            {
                treat(upRight, theNode, gridDestination);
            }
        }

        Point upLeft = new Point(openList.get(index).position.getX()-1, openList.get(index).position.getY()-1);
        if(upLeft.getX() >= 0 && upLeft.getY() >= 0) // in bounds of grid
        {
            if((w.getObjectsAt(toActual(upLeft).getX(), toActual(upLeft).getY(), Wall.class).isEmpty() || upLeft.is(gridDestination)) && (w.getObjectsAt(toActual(up).getX(), toActual(up).getY(), Wall.class).isEmpty() && w.getObjectsAt(toActual(left).getX(), toActual(left).getY(), Wall.class).isEmpty())) // unblocked
            {
                treat(upLeft, theNode, gridDestination);
            }
        }
    }
    public boolean isInOpen(Node node)//checks to see if the node's position matches any in the list
    {
        for(int n = 0; n < openList.size(); n ++)
        {
            if( openList.get(n).isOn(node.position) )
                return true;
        }
        return false;
    }
    public boolean isInClosed(Node node)
    {
        for(int n = 0; n < closedList.size(); n ++)
        {
            if(closedList.get(n).isOn(node.position))
            return true;
        }
        return false;
    }
    public boolean isInClosed(Point p)
    {
        for(int n = 0; n < closedList.size(); n ++)
        {
            if(closedList.get(n).isOn(p))
            return true;
        }
        return false;
    }
    public int indexInClosed(Point p)//returns the index where point is found in the closedList
    {
        for(int n = 0; n < closedList.size(); n ++)
        {
            if(closedList.get(n).isOn(p))
            {
                return n;
            }
        }
        return -1;
    }
    public int indexInOpen(Node node)
    {
        for(int n = 0; n < openList.size(); n ++)
        {
            if(openList.get(n).isOn(node.position))
            {
                return n;
            }
        }
        return -1;
    }
    public void setDestination(int x, int y)
    {
        destination.setPoint(x,y);
    }
}
